using System;
using Benday.FakesSample.Business;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.FakesSample.DataAccess;
using Microsoft.QualityTools.Testing.Fakes;
using Benday.FakesSample.Common;
using Benday.FakesSample.Common.Fakes;

namespace Benday.FakesSample.UnitTests
{
    [TestClass]
    public class PersonToDataRowAdapterFixture
    {
        [TestMethod]
        public void AdaptFromDataRowToIPerson()
        {
            var fromValue = new FakesDataSet().Person.NewPersonRow();

            PopulateWithTestData(fromValue);

            IPerson toValue = new StubIPerson();

            SystemUnderTest.Adapt(fromValue, toValue);

            AssertAreEqual(fromValue, toValue);
        }

        [TestMethod]
        public void AdaptFromIPersonToDataRow()
        {
            IPerson fromValue = new StubIPerson();

            PopulateWithTestData(fromValue);

            FakesDataSet.PersonRow toValue = new FakesDataSet().Person.NewPersonRow();

            SystemUnderTest.Adapt(fromValue, toValue);

            AssertAreEqual(fromValue, toValue);
        }

        [TestMethod]
        public void AdaptFromIPersonToDataRowDoesNotModifyIdIfAlreadyHasValue()
        {
            IPerson fromValue = new StubIPerson();

            PopulateWithTestData(fromValue);

            FakesDataSet.PersonRow toValue = new FakesDataSet().Person.NewPersonRow();

            PopulateWithTestData(toValue);

            int originalToValue = toValue.Id;

            SystemUnderTest.Adapt(fromValue, toValue);

            AssertAreEqual(fromValue, toValue, true);

            Assert.AreEqual<int>(originalToValue, toValue.Id, "toValue.Id should not change");
        }

        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
        }

        private PersonToDataRowAdapter _SystemUnderTest;
        public PersonToDataRowAdapter SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest = new PersonToDataRowAdapter();
                }

                return _SystemUnderTest;
            }
        }

        private void PopulateWithTestData(FakesDataSet.PersonRow value)
        {
            value.Id = 123;
            value.CreateDate = new DateTime(2013, 1, 1);
            value.EmailAddress = "benday@benday.com";
            value.FirstName = "Ben";
            value.LastName = "Day";
            value.Notes = "notes";
            value.LastModified = new DateTime(2014, 2, 1);
        }

        private void AssertAreEqual(FakesDataSet.PersonRow expected, IPerson actual)
        {
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.EmailAddress, actual.EmailAddress, "EmailAddress");
            Assert.AreEqual<string>(expected.FirstName, actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName, actual.LastName, "LastName");
            Assert.AreEqual<string>(expected.Notes, actual.Notes, "Notes");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
        }

        private void PopulateWithTestData(IPerson value)
        {
            value.Id = 234;
            value.CreateDate = new DateTime(2013, 1, 1);
            value.EmailAddress = "benday@benday.com";
            value.FirstName = "Ben";
            value.LastName = "Day";
            value.Notes = "notes";
            value.LastModified = new DateTime(2014, 2, 1);
        }

        private void AssertAreEqual(IPerson expected, FakesDataSet.PersonRow actual)
        {
            AssertAreEqual(expected, actual, false);
        }

        private void AssertAreEqual(IPerson expected, FakesDataSet.PersonRow actual, bool skipId)
        {
            if (skipId == false)
            {
                Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            }

            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.EmailAddress, actual.EmailAddress, "EmailAddress");
            Assert.AreEqual<string>(expected.FirstName, actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName, actual.LastName, "LastName");
            Assert.AreEqual<string>(expected.Notes, actual.Notes, "Notes");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
        }        
    }
}
